@@ -208,6 +208,19 @@ gpm_manager_is_policy_timout_valid (GpmManager *manager,
 	return TRUE;
 }
 
+/**
+ * gpm_manager_reset_event_time:
+ * @manager: This manager class instance
+ *
+ * Resets the time so we do not do any more actions until the
+ * timeout has passed.
+ **/
+static void
+gpm_manager_reset_event_time (GpmManager *manager)
+{
+	manager->priv->last_resume_event = time (NULL);
+}
+
 /**
  * gpm_manager_is_inhibit_valid:
  * @manager: This manager class instance
@@ -228,12 +241,13 @@ gpm_manager_is_inhibit_valid (GpmManager *manager,
 
 	action_ok = gpm_inhibit_check (manager->priv->inhibit);
 	if (! action_ok) {
+		GString *message = g_string_new ("");
+
 		if (manager->priv->ignore_inhibits) {
 			gpm_debug ("Inhibit ignored through gconf policy!");
 			return TRUE;
 		}
 		title = g_strdup_printf (_("Request to %s"), action);
-		GString *message = g_string_new ("");
 		gpm_inhibit_get_message (manager->priv->inhibit, message, action);
 		gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
 				      title,
@@ -325,8 +339,9 @@ gpm_manager_allowed_shutdown (GpmManager *manager,
 			      gboolean   *can,
 			      GError    **error)
 {
-	*can = FALSE;
 	gboolean polkit_ok = TRUE;
+	g_return_val_if_fail (can, FALSE);
+	*can = FALSE;
 #ifdef HAVE_POLKIT
 	polkit_ok = gpm_polkit_is_user_privileged (manager->priv->polkit, "hal-power-shutdown");
 #endif
@@ -348,8 +363,9 @@ gpm_manager_allowed_reboot (GpmManager *manager,
 			    gboolean   *can,
 			    GError    **error)
 {
-	*can = FALSE;
 	gboolean polkit_ok = TRUE;
+	g_return_val_if_fail (can, FALSE);
+	*can = FALSE;
 #ifdef HAVE_POLKIT
 	polkit_ok = gpm_polkit_is_user_privileged (manager->priv->polkit, "hal-power-reboot");
 #endif
@@ -531,39 +547,94 @@ sync_dpms_policy (GpmManager *manager)
 	GError  *error;
 	gboolean res;
 	gboolean on_ac;
+	guint    timeout;
 	guint    standby;
 	guint    suspend;
 	guint    off;
+	char    *dpms_method;
 
 	error = NULL;
 
 	gpm_power_get_on_ac (manager->priv->power, &on_ac, NULL);
 
 	if (on_ac) {
-		standby = gconf_client_get_int (manager->priv->gconf_client,
+		error = NULL;
+		timeout = gconf_client_get_int (manager->priv->gconf_client,
 						GPM_PREF_AC_SLEEP_DISPLAY,
 						&error);
+		if (error) {
+			gpm_warning ("Unable to get DPMS timeout: %s", error->message);
+			g_error_free (error);
+		}
+
+		error = NULL;
+		dpms_method = gconf_client_get_string (manager->priv->gconf_client,
+						       GPM_PREF_AC_DPMS_METHOD,
+						       &error);
+		if (error) {
+			gpm_warning ("Unable to get DPMS suspend method: %s", error->message);
+			g_error_free (error);
+		}
 	} else {
-		standby = gconf_client_get_int (manager->priv->gconf_client,
+		error = NULL;
+		timeout = gconf_client_get_int (manager->priv->gconf_client,
 						GPM_PREF_BATTERY_SLEEP_DISPLAY,
 						&error);
-	}
+		if (error) {
+			gpm_warning ("Unable to get DPMS timeout: %s", error->message);
+			g_error_free (error);
+		}
 
-	if (error) {
-		gpm_warning ("Unable to get DPMS timeouts: %s", error->message);
-		g_error_free (error);
-		return;
+		error = NULL;
+		dpms_method = gconf_client_get_string (manager->priv->gconf_client,
+						       GPM_PREF_BATTERY_DPMS_METHOD,
+						       &error);
+		if (error) {
+			gpm_warning ("Unable to get DPMS suspend method: %s", error->message);
+			g_error_free (error);
+		}
 	}
 
 	/* old policy was in seconds, warn the user if too small */
-	if (standby < 60) {
+	if (timeout < 60) {
 		gpm_warning ("standby timeout is invalid, please re-configure");
 		return;
 	}
 
-	/* try to make up some reasonable numbers */
-	suspend = standby;
-	off     = standby * 2;
+	/* If we have no dpms_method, possible due to a schema problem */
+	if (dpms_method == NULL) {
+		g_warning ("DPMS method unknown. Possible schema problem!");
+		return;
+	}
+
+	/* Some monitors do not support certain suspend states, so we have to
+	 * provide a way to only use the one that works. */
+	if (strcmp (dpms_method, "default") == 0) {
+		/* suspend after one timeout, turn off after another */
+		standby = timeout;
+		suspend = timeout;
+		off     = timeout * 2;
+	} else if (strcmp (dpms_method, "standby") == 0) {
+		standby = timeout;
+		suspend = 0;
+		off     = 0;
+	} else if (strcmp (dpms_method, "suspend") == 0) {
+		standby = 0;
+		suspend = timeout;
+		off     = 0;
+	} else if (strcmp (dpms_method, "off") == 0) {
+		standby = 0;
+		suspend = 0;
+		off     = timeout;
+	} else {
+		/* wtf? */
+		g_warning ("unknown dpms mode!");
+		g_free (dpms_method);
+		return;
+	}
+
+	gpm_debug ("DPMS parameters %d %d %d, method '%s'\n", standby, suspend, off, dpms_method);
+	g_free (dpms_method);
 
 	error = NULL;
 	res = gpm_dpms_set_enabled (manager->priv->dpms, TRUE, &error);
@@ -664,8 +735,11 @@ change_power_policy (GpmManager *manager,
 		power_save = gconf_client_get_bool (client, GPM_PREF_BATTERY_LOWPOWER, NULL);
 	}
 
-	gpm_brightness_set_level_std (manager->priv->brightness, brightness);
-	gpm_brightness_set (manager->priv->brightness);
+	/* only do brightness changes if we have the hardware */
+	if (gpm_brightness_has_hardware (manager->priv->brightness)) {
+		gpm_brightness_set_level_std (manager->priv->brightness, brightness);
+		gpm_brightness_set (manager->priv->brightness);
+	}
 
 	gpm_hal_enable_power_save (manager->priv->hal, power_save);
 	update_ac_throttle (manager, on_ac);
@@ -726,6 +800,7 @@ gpm_manager_blank_screen (GpmManager *manager,
 {
 	gboolean do_lock;
 	gboolean ret = TRUE;
+	GError  *error = NULL;
 
 	gpm_info_event_log (manager->priv->info, GPM_GRAPH_EVENT_DPMS_OFF, NULL);
 
@@ -735,7 +810,6 @@ gpm_manager_blank_screen (GpmManager *manager,
 		if (!gpm_screensaver_lock (manager->priv->screensaver))
 			gpm_debug ("Could not lock screen via gnome-screensaver");
 	}
-	GError     *error = NULL;
 	gpm_dpms_set_mode (manager->priv->dpms, GPM_DPMS_MODE_OFF, &error);
 	if (error) {
 		gpm_debug ("Unable to set DPMS mode: %s", error->message);
@@ -899,14 +973,15 @@ gpm_manager_get_low_power_mode (GpmManager  *manager,
 				gboolean    *retval,
 				GError     **error)
 {
+	gboolean on_ac;
+	gboolean power_save;
+
 	g_return_val_if_fail (GPM_IS_MANAGER (manager), FALSE);
 
 	if (retval == NULL) {
 		return FALSE;
 	}
 
-	gboolean on_ac;
-	gboolean power_save;
 	gpm_power_get_on_ac (manager->priv->power, &on_ac, error);
 	if (on_ac) {
 		power_save = gconf_client_get_bool (manager->priv->gconf_client,
@@ -1092,7 +1167,7 @@ gpm_manager_reboot (GpmManager *manager,
  **/
 static gboolean
 gpm_manager_hibernate (GpmManager *manager,
-			GError    **error)
+		       GError    **error)
 {
 	gboolean allowed;
 	gboolean ret;
@@ -1127,14 +1202,19 @@ gpm_manager_hibernate (GpmManager *manager,
 	if (! ret) {
 		char *message;
 		gboolean show_notify;
+
+		if (manager->priv->enable_beeping) {
+			gpm_warning_beep ();
+		}
+
 		/* We only show the HAL failed notification if set in gconf */
 		show_notify = gconf_client_get_bool (manager->priv->gconf_client,
 						     GPM_PREF_NOTIFY_HAL_ERROR, NULL);
 		if (show_notify) {
+			const char *title = _("Hibernate Problem");
 			message = g_strdup_printf (_("HAL failed to %s. "
 						     "Check the <a href=\"%s\">FAQ page</a> for common problems."),
 						     _("hibernate"), GPM_FAQ_URL);
-			const char *title = _("Hibernate Problem");
 			gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
 					      title,
 					      message,
@@ -1155,7 +1235,7 @@ gpm_manager_hibernate (GpmManager *manager,
 	sync_dpms_policy (manager);
 
 	/* save the time that we resumed */
-	manager->priv->last_resume_event = time (NULL);
+	gpm_manager_reset_event_time (manager);
 
 	return ret;
 }
@@ -1179,6 +1259,8 @@ gpm_manager_suspend (GpmManager *manager,
 	gboolean do_lock;
 	GpmPowerStatus status;
 	char *message;
+	int charge_before_suspend;
+	int charge_difference;
 
 	gpm_manager_allowed_suspend (manager, &allowed, NULL);
 
@@ -1204,7 +1286,7 @@ gpm_manager_suspend (GpmManager *manager,
 	gpm_power_get_battery_status (manager->priv->power,
 				      GPM_POWER_KIND_PRIMARY,
 				      &status);
-	int charge_before_suspend = status.current_charge;
+	charge_before_suspend = status.current_charge;
 
 	/* Do the suspend */
 	ret = gpm_hal_suspend (manager->priv->hal, 0);
@@ -1218,7 +1300,7 @@ gpm_manager_suspend (GpmManager *manager,
 	gpm_power_get_battery_status (manager->priv->power,
 				      GPM_POWER_KIND_PRIMARY,
 				      &status);
-	int charge_difference = status.current_charge - charge_before_suspend;
+	charge_difference = status.current_charge - charge_before_suspend;
 	if (charge_difference != 0) {
 		if (charge_difference > 0) {
 			message = g_strdup_printf (_("Battery charged %imWh during suspend"),
@@ -1233,26 +1315,34 @@ gpm_manager_suspend (GpmManager *manager,
 		g_free (message);
 	}
 
-	gboolean show_notify;
-	/* We only show the HAL failed notification if set in gconf */
-	show_notify = gconf_client_get_bool (manager->priv->gconf_client,
-					     GPM_PREF_NOTIFY_HAL_ERROR, NULL);
-	if ((!ret) && show_notify) {
-		const char *title;
-		message = g_strdup_printf (_("Your computer failed to %s.\n"
-					     "Check the <a href=\"%s\">FAQ page</a> for common problems."),
-					     _("suspend"), GPM_FAQ_URL);
-		title = _("Suspend Problem");
-		gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
-				      title,
-				      message,
-				      GPM_NOTIFY_TIMEOUT_LONG,
-				      GTK_STOCK_DIALOG_WARNING,
-				      GPM_NOTIFY_URGENCY_LOW);
-		gpm_info_event_log (manager->priv->info,
-				    GPM_GRAPH_EVENT_NOTIFICATION,
-				    title);
-		g_free (message);
+	if (! ret) {
+		char *message;
+		gboolean show_notify;
+
+		if (manager->priv->enable_beeping) {
+			gpm_warning_beep ();
+		}
+
+		/* We only show the HAL failed notification if set in gconf */
+		show_notify = gconf_client_get_bool (manager->priv->gconf_client,
+						     GPM_PREF_NOTIFY_HAL_ERROR, NULL);
+		if (show_notify) {
+			const char *title;
+			message = g_strdup_printf (_("Your computer failed to %s.\n"
+						     "Check the <a href=\"%s\">FAQ page</a> for common problems."),
+						     _("suspend"), GPM_FAQ_URL);
+			title = _("Suspend Problem");
+			gpm_tray_icon_notify (GPM_TRAY_ICON (manager->priv->tray_icon),
+					      title,
+					      message,
+					      GPM_NOTIFY_TIMEOUT_LONG,
+					      GTK_STOCK_DIALOG_WARNING,
+					      GPM_NOTIFY_URGENCY_LOW);
+			gpm_info_event_log (manager->priv->info,
+					    GPM_GRAPH_EVENT_NOTIFICATION,
+					    title);
+			g_free (message);
+		}
 	}
 
 	if (do_lock) {
@@ -1263,7 +1353,7 @@ gpm_manager_suspend (GpmManager *manager,
 	sync_dpms_policy (manager);
 
 	/* save the time that we resumed */
-	manager->priv->last_resume_event = time (NULL);
+	gpm_manager_reset_event_time (manager);
 
 	return ret;
 }
@@ -1370,7 +1460,7 @@ idle_changed_cb (GpmIdle    *idle,
 		/* Should we resume the screen? */
 		do_laptop_dim = gconf_client_get_bool (manager->priv->gconf_client,
 						       GPM_PREF_IDLE_DIM_SCREEN, NULL);
-		if (do_laptop_dim) {
+		if (do_laptop_dim && gpm_brightness_has_hardware (manager->priv->brightness)) {
 			/* resume to the previous brightness */
 			manager_explain_reason (manager, GPM_GRAPH_EVENT_SCREEN_RESUME,
 						_("Screen resume"),
@@ -1396,7 +1486,7 @@ idle_changed_cb (GpmIdle    *idle,
 		/* Should we dim the screen? */
 		do_laptop_dim = gconf_client_get_bool (manager->priv->gconf_client,
 						       GPM_PREF_IDLE_DIM_SCREEN, NULL);
-		if (do_laptop_dim) {
+		if (do_laptop_dim && gpm_brightness_has_hardware (manager->priv->brightness)) {
 			/* Dim the screen, fixes #328564 */
 			manager_explain_reason (manager, GPM_GRAPH_EVENT_SCREEN_DIM,
 						_("Screen dim"),
@@ -1678,11 +1768,17 @@ power_on_ac_changed_cb (GpmPower   *power,
 			gboolean    on_ac,
 			GpmManager *manager)
 {
+	gboolean event_when_closed;
+
 	gpm_debug ("Setting on-ac: %d", on_ac);
 
 	/* simulate user input, to fix #333525 */
 	gpm_screensaver_poke (manager->priv->screensaver);
 
+	/* Don't do any events for a few seconds after we remove the
+	 * ac_adapter. See #348201 for details */
+	gpm_manager_reset_event_time (manager);
+
 	/* If we are on AC power we should show warnings again */
 	if (on_ac) {
 		gpm_debug ("Resetting warning to NONE as on AC power");
@@ -1712,7 +1808,6 @@ power_on_ac_changed_cb (GpmPower   *power,
 
 	/* We do the lid close on battery action if the ac_adapter is removed
 	   when the laptop is closed and on battery. Fixes #331655 */
-	gboolean event_when_closed;
 	event_when_closed = gconf_client_get_bool (manager->priv->gconf_client,
 						   GPM_PREF_BATT_EVENT_WHEN_CLOSED, NULL);
 	if (event_when_closed && (!on_ac) && manager->priv->lid_is_closed) {
@@ -1895,7 +1990,8 @@ battery_status_changed_primary (GpmManager     *manager,
 	/* We only re-enable the fully charged notification when the battery
 	   drops down to 95% as some batteries charge to 100% and then fluctuate
 	   from ~98% to 100%. See #338281 for details */
-	if (battery_status->percentage_charge < 95) {
+	if (battery_status->percentage_charge < 95 &&
+	    gpm_power_battery_is_charged (battery_status) == FALSE) {
 		manager->priv->done_notify_fully_charged = FALSE;
 	}
 
@@ -2330,6 +2426,14 @@ gconf_key_changed_cb (GConfClient *client,
 
 		sync_dpms_policy (manager);
 
+	} else if (strcmp (entry->key, GPM_PREF_AC_DPMS_METHOD) == 0) {
+
+		sync_dpms_policy (manager);
+
+	} else if (strcmp (entry->key, GPM_PREF_BATTERY_DPMS_METHOD) == 0) {
+
+		sync_dpms_policy (manager);
+
 	} else if (strcmp (entry->key, GPM_PREF_AC_BRIGHTNESS) == 0) {
 
 		if (on_ac) {
@@ -2485,6 +2589,7 @@ tray_icon_destroyed (GtkObject *object, gpointer user_data)
 /**
  * screensaver_auth_request_cb:
  * @manager: This manager class instance
+ * @auth: If we are trying to authenticate
  *
  * Undim the screen when the login screen appears (see #333290)
  **/
@@ -2493,12 +2598,30 @@ screensaver_auth_request_cb (GpmScreensaver *screensaver,
 			     gboolean        auth,
 			     GpmManager     *manager)
 {
+	GError  *error;
+	gboolean res;
+
+	/* Only act on the begin authentication request */
+	if (! auth) {
+		return;
+	}
+
 	/* TODO: This may be a bid of a bodge, as we will have multiple
 		 resume requests -- maybe this need a logic cleanup */
-	if (auth) {
-		gpm_debug ("resuming due to auth begin");
+	if (gpm_brightness_has_hardware (manager->priv->brightness)) {
+		gpm_debug ("undimming lcd due to auth begin");
 		gpm_brightness_undim (manager->priv->brightness);
 	}
+
+	/* We turn on the monitor unconditionally, as we may be using
+	 * a smartcard to authenticate and DPMS might still be on.
+	 * See #350291 for more details */
+	error = NULL;
+	res = gpm_dpms_set_mode (manager->priv->dpms, GPM_DPMS_MODE_ON, &error);
+	if (! res) {
+		gpm_warning ("Failed to turn on DPMS: %s", error->message);
+		g_error_free (error);
+	}
 }
 
 /**
@@ -2535,6 +2658,22 @@ hal_daemon_monitor_cb (GpmHal     *hal,
 	tray_icon_update (manager);
 }
 
+/**
+ * manager_rescan_buttons:
+ * @manager: This manager class instance
+ *
+ * Rescan the buttons to avoid #346082
+ *
+ * Return value: TRUE, as we want to repeat this action on resume.
+ **/
+static gboolean
+manager_rescan_buttons (GpmManager *manager)
+{
+	g_debug ("rescanning buttons");
+	gpm_hal_device_rescan_capability (manager->priv->hal, "button");
+	return TRUE;
+}
+
 /**
  * gpm_manager_init:
  * @manager: This manager class instance
@@ -2547,6 +2686,7 @@ gpm_manager_init (GpmManager *manager)
 	gboolean check_type_cpu;
 	gboolean enabled;
 	gboolean allowed_in_menu;
+	int lcd_dim_brightness;
 
 	manager->priv = GPM_MANAGER_GET_PRIVATE (manager);
 
@@ -2696,7 +2836,7 @@ gpm_manager_init (GpmManager *manager)
 		   manager->priv->suppress_policy_timeout);
 
 	/* Pretend we just resumed when we start to let actions settle */
-	manager->priv->last_resume_event = time (NULL);
+	gpm_manager_reset_event_time (manager);
 
 	/* get percentage policy */
 	manager->priv->low_percentage = gconf_client_get_int (manager->priv->gconf_client,
@@ -2725,10 +2865,12 @@ gpm_manager_init (GpmManager *manager)
 							   GPM_PREF_ACTION_TIME, NULL);
 
 	/* Get dim settings */
-	int lcd_dim_brightness;
-	lcd_dim_brightness = gconf_client_get_int (manager->priv->gconf_client,
-						   GPM_PREF_PANEL_DIM_BRIGHTNESS, NULL);
-	gpm_brightness_set_level_dim (manager->priv->brightness, lcd_dim_brightness);
+	if (gpm_brightness_has_hardware (manager->priv->brightness)) {
+		lcd_dim_brightness = gconf_client_get_int (manager->priv->gconf_client,
+							   GPM_PREF_PANEL_DIM_BRIGHTNESS, NULL);
+		gpm_debug ("lcd_dim_brightness is %i", lcd_dim_brightness);
+		gpm_brightness_set_level_dim (manager->priv->brightness, lcd_dim_brightness);
+	}
 
 	/* Do we beep? */
 	manager->priv->enable_beeping = gconf_client_get_bool (manager->priv->gconf_client,
@@ -2737,6 +2879,9 @@ gpm_manager_init (GpmManager *manager)
 	/* Do we ignore inhibit requests? */
 	manager->priv->ignore_inhibits = gconf_client_get_bool (manager->priv->gconf_client,
 								GPM_PREF_IGNORE_INHIBITS, NULL);
+
+	/* poll the lid periodically to avoid #346082 */
+	g_timeout_add (1000 * 60, (GSourceFunc) manager_rescan_buttons, manager);
 }
 
 /**
